/* $Id: ppc.c,v 1.12 2004/11/24 00:39:15 mikpe Exp $
 * PPC32-specific perfctr library procedures.
 *
 * Copyright (C) 2004  Mikael Pettersson
 */
#include <errno.h>
#include <asm/unistd.h>
#include <stdio.h>
#include "libperfctr.h"
#include "ppc.h"

#ifndef __NR_perfctr_info
#define __NR_perfctr_info	272
#define __NR_vperfctr_open	(__NR_perfctr_info+1)
#define __NR_vperfctr_control	(__NR_perfctr_info+2)
#define __NR_vperfctr_unlink	(__NR_perfctr_info+3)
#define __NR_vperfctr_iresume	(__NR_perfctr_info+4)
#define __NR_vperfctr_read	(__NR_perfctr_info+5)
#endif

#include <unistd.h>

int _sys_perfctr_info(int fd_unused, struct perfctr_info *info, struct perfctr_cpu_mask *cpus, struct perfctr_cpu_mask *forbidden)
{
    return syscall(__NR_perfctr_info, info, cpus, forbidden);
}

int _sys_vperfctr_open(int fd_unused, int tid, int creat)
{
    return syscall(__NR_vperfctr_open, tid, creat);
}

int _sys_vperfctr_control(int fd, const struct vperfctr_control *arg)
{
    return syscall(__NR_vperfctr_control, fd, arg, sizeof *arg);
}

int _sys_vperfctr_unlink(int fd)
{
    return syscall(__NR_vperfctr_unlink, fd);
}

int _sys_vperfctr_iresume(int fd)
{
    return syscall(__NR_vperfctr_iresume, fd);
}

int _sys_vperfctr_read(int fd, unsigned int cmd, void *argp, unsigned int argbytes)
{
    return syscall(__NR_vperfctr_read, fd, cmd, argp, argbytes);
}

void perfctr_info_cpu_init(struct perfctr_info *info)
{
    unsigned int pvr = mfspr(SPRN_PVR); /* trapped & emulated by the kernel */
    unsigned int cpu_type;

    switch( PVR_VER(pvr) ) {
      case 0x0004: /* 604 */
	cpu_type = PERFCTR_PPC_604;
	break;
      case 0x0009: /* 604e */
      case 0x000A: /* 604ev */
	cpu_type = PERFCTR_PPC_604e;
	break;
      case 0x0008: /* 750/740 */
      case 0x7000: case 0x7001: /* 750FX */
      case 0x7002: /* 750GX */
	cpu_type = PERFCTR_PPC_750;
	break;
      case 0x000C: /* 7400 */
      case 0x800C: /* 7410 */
	cpu_type = PERFCTR_PPC_7400;
	break;
      case 0x8000: /* 7451/7441 */
      case 0x8001: /* 7455/7445 */
      case 0x8002: /* 7457/7447 */
      case 0x8003: /* 7447A */
      case 0x8004: /* 7448 */
	cpu_type = PERFCTR_PPC_7450;
	break;
      default:
	cpu_type = PERFCTR_PPC_GENERIC;
    }
    info->cpu_type = cpu_type;
}

unsigned int perfctr_info_nrctrs(const struct perfctr_info *info)
{
    switch( info->cpu_type ) {
      case PERFCTR_PPC_604:
	return 2;
      case PERFCTR_PPC_604e:
      case PERFCTR_PPC_750:
      case PERFCTR_PPC_7400:
	return 4;
      case PERFCTR_PPC_7450:
	return 6;
      default:
	return 0;
    }
}

const char *perfctr_info_cpu_name(const struct perfctr_info *info)
{
    switch( info->cpu_type ) {
      case PERFCTR_PPC_GENERIC:
	return "Generic PowerPC with TB";
      case PERFCTR_PPC_604:
	return "PowerPC 604";
      case PERFCTR_PPC_604e:
	return "PowerPC 604e";
      case PERFCTR_PPC_750:
	return "PowerPC 750";
      case PERFCTR_PPC_7400:
	return "PowerPC 7400";
      case PERFCTR_PPC_7450:
	return "PowerPC 7450";
      default:
	return "?";
    }
}

void perfctr_cpu_control_print(const struct perfctr_cpu_control *control)
{
    unsigned int i, nractrs, nrictrs, nrctrs;

    nractrs = control->nractrs;
    nrictrs = control->nrictrs;
    nrctrs = control->nractrs + nrictrs;

    printf("tsc_on\t\t\t%u\n", control->tsc_on);
    printf("nractrs\t\t\t%u\n", nractrs);
    if( nrictrs )
	printf("nrictrs\t\t\t%u\n", nrictrs);
    for(i = 0; i < nrctrs; ++i) {
	printf("pmc[%u].map\t\t%u\n", i, control->pmc[i].map);
        printf("pmc[%u].evntsel\t\t0x%08X\n", i, control->pmc[i].evntsel);
	if( i >= nractrs )
	    printf("pmc[%u].ireset\t\t%d\n", i, control->pmc[i].ireset);
    }
    if( control->ppc.mmcr0 )
	printf("mmcr0\t\t\t0x%08X\n", control->ppc.mmcr0);
    if( control->ppc.mmcr2 )
	printf("mmcr2\t\t\t0x%08X\n", control->ppc.mmcr2);
}
